#ifndef mash_session_scuba_h
#define mash_session_scuba_h

#include "net.h"
#include "timer.h"

class ScubaSessionManager;

class ScubaHandler : public DataHandler, public Timer {
    public:
	ScubaHandler();
	virtual void dispatch(int mask);
	virtual void net(Network* net);
	virtual void timeout();
	void adapt(int size);
	void sample_size(int cc);
	inline double rint() const { return (rint_); }
	inline void bandwidth(double bw) { bw_ = bw; ctrl_inv_bw_ = 1. / bw; }
	inline double bandwidth() { return (bw_); }
	inline void manager(ScubaSessionManager* m) { manager_ = m; }
	void startctrl();
 protected:
	void schedule_timer();
	double bw_;
	double ctrl_inv_bw_;
	double ctrl_avg_size_;	/* (estimated) average size of ctrl packets */
	double rint_;		/* current session report rate (in ms) */

	ScubaSessionManager* manager_;
};

class ScubaSessionManager {
public:
	virtual void recv(ScubaHandler*) = 0;
	virtual void announce(ScubaHandler*) = 0;
};

/*
 * Parameters controlling the SCUBA report rate timer.
 */
#define SCUBA_SESSION_BW_FRACTION (0.05)
#define SCUBA_MIN_RPT_TIME (1.)
#define SCUBA_SENDER_BW_FRACTION (0.25)
#define SCUBA_RECEIVER_BW_FRACTION (1. - SCUBA_SENDER_BW_FRACTION)
#define SCUBA_SIZE_GAIN (1./8.)

#endif
	
	
