#
# Copyright (c) 1993-1995 Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by the Network Research
#	Group at Lawrence Berkeley National Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/mash/mash-1/tcl/hvic/ui-headless.tcl,v 1.22 1998/05/05 13:52:16 cromer Exp $ (LBL)
#

import RTPAgent VisualFrame

#
# Make an agent that monitors RTP audio activity
#
Class AudioMonitor -superclass RTPAgent

#XXX
AudioMonitor private create_session {} {
	return [new Session/RTP/Audio]
}

#
AudioMonitor instproc init { ab ui } {
	$self next $ab
	$self set ui_ $ui
}

#
AudioMonitor instproc activate src {
	$self instvar null_
	if ![info exists null_] {
		set null_ [new Module/AudioDecoder]
	}
	$src handler $null_
	$self next $src
#	puts "activate [$src getid]"
}

#
AudioMonitor instproc trigger_media src {
	#XXX race: if src disappears will get an undefined command
	after 500 "$src enable_trigger"
	$self next $src
	$self instvar ui_
	
	$ui_ switch_to $src
}

#
AudioMonitor instproc create-session sessionType {
	if { $sessionType == "rtpv2" || $sessionType == "rtp" } {
		set sessionType RTP
	}
	return [new Session/$sessionType/Audio]
}

import VicUI

#
# The main user interface for the hvic (headless vic) application.
#
Class HeadlessVicUI -superclass VicUI

#XXX act as ControlMenu...      
HeadlessVicUI instproc use-hw { } {
	return 0
}

#
# Build the user-interface.
#
HeadlessVicUI instproc build_gui { w } {

	$self instvar userwindows_ grid_ label_ controlMenu_ path_ window_ vframe_ 
	set path_ $w

	set vframe_ [new VisualFrame $w.top $self]

	$self set allsrcs_ ""
	set p $w.top.vw
	#XXX
#	set window_ [new VideoWidget $p 640 480 $self]
	set geometry [split [$self get_option vwGeometry] x]
	if { $geometry == "" } {
		set geometry [split [$self get_option geometry] x] 
		if { $geometry == "" } {
			set geometry "320 240"
		}
	}
	set window_ [eval new VideoWidget $p $geometry]
	pack $w.top.vw
	bind . <s> "$self switch_src"
	bind . <Return> "$self switch_src"
	bind . <q> "exit 1"
	pack $w.top
        
	set controlMenu_ [$self mk_ctrlmenu $w]
        
	set aspec 224.2.253.23/61172
	#XXX
	# puts stderr "warning audio-mon address hardwired to: $aspec"

	if { $aspec != "" } {
		set ab [new AddressBlock $aspec]
		set monitor [new AudioMonitor $ab $self]
		# audio doesn't want traffic looped back
		#$audioAgent_ set-loopback 0
		# send back-to-back packets spaced out at 128kb/s
		#$audioAgent_ set-bandwidth 128
	}
	after 1000 "$self periodic_update"
}

#
# Create the ControlMenu to serve as a user interface
# to the <i>videoAgent_</i> and VideoPipeline, <i>vpipe_</i>.
#
HeadlessVicUI instproc mk_ctrlmenu { w } {
	$self instvar videoAgent_ vpipe_ vframe_

	# XXX sending {} for the ActiveSourceManager parameter, which will likely only cause problems if you try to redecorate using build.tile
	return [new ControlMenu $self $videoAgent_ $vpipe_ $vframe_ {} [new UISrcListWindow $w $videoAgent_]]
}

#
HeadlessVicUI instproc switch_src {} {
	$self instvar allsrcs_ attached_src_ active_ window_
	if { [llength $allsrcs_] > 0 } {
		set len [llength $allsrcs_]
		set i [lsearch $allsrcs_ $attached_src_]
		set i [expr ($i+1) % $len]
		set olds $active_($attached_src_)
		$olds detach-window $window_
		set src [lindex $allsrcs_ $i]
		set as $active_($src)
		$as attach-window $window_
		set attached_src_ $src
		$window_ redraw
	}
}

#
# <i>src</i> is an audio src. <br>
# <i>allsrcs_</i> are video sources
#
HeadlessVicUI instproc switch_to src {
	$self instvar allsrcs_ attached_src_ window_ active_
	if { [string match *CoLab* [$src getid]] } {
		return
	}
	if { [llength $allsrcs_] > 0 } {
		set target ""
		foreach s $allsrcs_ {
			if { "[$src sdes cname]" == "[$s sdes cname]" } {
				set target $s
			}
		}
		if { $target == "" } {
			puts stderr "no target"
			return
		}
puts "switch to [$target getid]"
		set olds $active_($attached_src_)
		$olds detach-window $window_
		set as $active_($target)
		$as attach-window $window_
		set attached_src_ $target
		$window_ redraw
	}
}

#
# Return a list of the active sources.
#
HeadlessVicUI instproc active-sources {} {
	$self instvar active_
	return [array names active_]
}

#
# Add the ActiveSource, <i>as</i>, to the <i>active_</i> array (indexed by Source, <i>src</i>).
# Also add the Source to the list, <i>allsrcs_</i>.
#
HeadlessVicUI instproc add_active { as src } {
	$self instvar active_ allsrcs_
	set active_($src) $as
	lappend allsrcs_ $src
}

#
# Remove the Source, <i>src</i>, from the <i>active_</i> array and the <i>allsrcs_</i> list.
#
HeadlessVicUI instproc rm_active src {
	$self instvar active_ allsrcs_
	unset active_($src)
	set i [lsearch $allsrcs_ $src]
	set allsrcs_ [lreplace allsrcs_ $i $i]
}

#
# Update the source descprtion by updating the element of the global
# arrays src_info(), src_nickname(), and src_name() indexed by the
# specified <i>src</i>.  If the src_name() is changed, then this is
# reflected in the UserWindows in which this src appears.
#
HeadlessVicUI instproc trigger_sdes src {
	#XXX
	return
	
	global src_info src_nickname src_name
	#
	# Figure out best presentation from available information.
	#
	set name [$src sdes name]
	set cname [$src sdes cname]
	set addr [$src addr]
	if { $name == "" } {
		if { $cname == "" } {
			set src_nickname($src) $addr
			set info $addr/[$src format_name]

		} else {
			set src_nickname($src) $cname
			set info "$addr/[$src format_name]"
		}
	} elseif [cname_redundant $name $cname] {
		set src_nickname($src) $name
		set info $addr/[$src format_name]
	} else {
		set src_nickname($src) $name
		set info $cname/[$src format_name]
	}
		set src_info($src) $cname/[$src format_name]

	set msg [$src sdes note]
	if { $msg != "" } {
		set info $msg
	}
	set src_info($src) $info
	
	# only call change_name when name really changes
	if { ![info exists src_name($src)] || "$src_name($src)" != "$name" } {
		set src_name($src) $name
		# XXX this will no longer work, because HeadlessVicUI does not implement "change_name"
		#  It is now implemented by ActiveSourceManager, which hvic doesn't even use.
#		$self change_name $src
	}
}

#
Class SimpleSource

#
SimpleSource instproc init { parent src } {
	$self next
	$self instvar src_ parent_
	set src_ $src
	set parent_ $parent
}

#
# bind a source to a window so that the video stream from
# source $src appears in window $vw, which is a UserWindow
#
SimpleSource instproc attach-window vw {
	$self instvar src_ parent_
	#$vw attach-decoder $src_ [[$parent_ set vframe_] set colorModel_] [$parent_ set controlMenu_]
	$vw attach-decoder $src_ [[$parent_ set vframe_] set colorModel_] [$parent_ use-hw]
}

#
SimpleSource instproc detach-window vw {
	$self instvar src_
	$vw detach-decoder $src_
}

# nop
HeadlessVicUI instproc trigger_media src {}

#
# Add a src to the active senders list.  E.g., make a postage
# stamp window appear, stats, etc. so that the user can select
# the video stream.
#
HeadlessVicUI instproc activate src {
	#
	# give the VideoAgent a chance to create and install
	# a decoder and for that decoder to see a packet so it can
	# determine the output geometry and color decimation.
	# we shouldn't have to do this (e.g., resize will
	# take care of a geometry change), but currently
	# decoders can't trigger a renderer realloation
	# when the decimation changes.XXX fix this
	#
	after idle "$self really_activate $src"
}

#
HeadlessVicUI instproc really_activate src {
	set as [new SimpleSource $self $src]
	$self add_active $as $src
	#XXX only if empty
	$self instvar window_ attached_src_
	if ![info exists attached_src_] {
		$as attach-window $window_
		set attached_src_ $src
	}
}

#
HeadlessVicUI instproc clear {} {
	$self instvar attached_src_ active_ window_
	if ![info exists attached_src_] {
		return
	}
	set as $active_($attached_src_)
	$as detach-window $window_
	unset attached_src_
}

#
HeadlessVicUI instproc resize { w h } {
	$self instvar attached_src_ window_ active_ path_
	if ![info exists attached_src_] {
		return
	}
	set as $active_($attached_src_)
	$as detach-window $window_
	[$window_ window] resize $w $h
	#
	# Force an update so the window gets mapped at
	# the new size before we re-bind the window
	# to the source.
	#
	update idletasks

	$as attach-window $window_
}

#
HeadlessVicUI instproc trigger_format src {
	$self instvar active_ videoAgent_ window_
	if ![info exists active_($src)] {
		#
		# if we get a change format before really_activate
		# was called (i.e., so we don't even have a thumbnail yet),
		# don't do anything
		#
		return
	}
	set as $active_($src)
	$as detach-window $window_
	set d [$src handler]
	delete $d
	set d [$videoAgent_ create_decoder $src]
	$as attach-window $window_
	$window_ redraw
}

#
# Called when the video stream state changes in a way that would
# affect the choice of renderer.  For example, when a jpeg stream
# changes from type-0 to type-1 we might have to revert from
# hardware to software decoding, or we might have to reallocate
# a 422 renderer as a 411 renderer.  This never needs to happen
# for most stream types (i.e., because the decimation factor is fixed).
#
HeadlessVicUI instproc decoder_changed src {
	$self instvar active_
	if ![info exists active_($src)] {
		return
	}
	#XXX redundant with trigger_format
	set as $active_($src)

	set L [$as user-windows]
	$as detach-windows
	#XXX
	set extoutList [extout_detach_src $src]
	foreach w $iw {
		$as attach-window $uw
		$w redraw
	}
	#XXX
	extout_attach_src $src $extoutList
	return
}

#
# Remove a src from the active senders list. 
#
HeadlessVicUI instproc deactivate src {
	$self instvar active_
	if [info exists active_($src)] {
		set as $active_($src)
		set L [$as user-windows]
		foreach uw $L {
			#XXX should check if we're voice-switched
			# and if so, bump window
			delete $uw
		}
		# detach any we missed by deleting user-windows
		$as detach-windows
	}

	$src handler ""
}

