
# This file defines the Parameter class and many subclasses.
# Each subclass of Parameter must provide implementations for
# the following methods:
#
# get
# set 
# type
# domain

Class Parameter;

Parameter instproc init args {
    eval $self next $args;
}

Class RealParameter -superclass Parameter

RealParameter instproc init args {
    $self instvar from_ to_ current_;
    set from_ 0.0;
    set to_ 1.0;
    set current_ 0.0;
    eval $self next $args;
}

RealParameter instproc from args {
    $self instvar from_;

    if {[llength $args] == 1} {
	set from_ $args;
    }
    return $from_;
}

RealParameter instproc to args {
    $self instvar to_;

    if {[llength $args] == 1} {
	set to_ $args;
    }
    return $to_;
}

RealParameter instproc get {} {
    $self instvar current_;

    return $current_;
}

RealParameter instproc set value {
    $self instvar current_ from_ to_;

    if {$value < $from_} {
	set current_ $from_;
    } elseif {$value > $to_} {
	set current_ $to_;
    } else {
	set current_ $value;
    }
    return $current_;
}

RealParameter instproc type {} {
    return "real";
}

RealParameter instproc domain {} {
    $self instvar from_ to_;

    return "($from_,$to_)"
}

Class IntParameter -superclass Parameter

IntParameter instproc init args {
    $self instvar from_ to_ current_;
    
    set from_ 0;
    set to_ 1;
    set current_ 0;
    eval $self next $args;
}

IntParameter instproc from args {
    $self instvar from_;

    if {[llength $args] == 1} {
	set from_ [expr int($args)];
    }
    return $from_;
}

IntParameter instproc to args {
    $self instvar to_;

    if {[llength $args] == 1} {
	set to_ [expr int($args)];
    }
    return $to_;
}

IntParameter instproc get {} {
    $self instvar current_;

    return $current_;
}

IntParameter instproc set value {
    $self instvar current_ from_ to_;

    if {$value < $from_} {
	set current_ $from_;
    } elseif {$value > $to_} {
	set current_ $to_;
    } else {
	set current_ [expr int($value)];
    }
    return $current_;
}

IntParameter instproc type {} {
    return "int";
}

IntParameter instproc domain {} {
    $self instvar from_ to_;

    return "($from_,$to_)"
}

Class TextParameter -superclass Parameter

TextParameter instproc init args {
    $self instvar current_;
    set current_ "";
    eval $self next $args;
}

TextParameter instproc get {} {
    $self instvar current_;

    return $current_;
}

TextParameter instproc set value {
    $self instvar current_;

    set current_ $value;
    return $current_;
}

TextParameter instproc type {} {
    return "text";
}

TextParameter instproc domain {} {
    return "all"
}

Class ExclusiveChoiceParameter -superclass Parameter

ExclusiveChoiceParameter instproc init args {
    $self instvar current_;
    $self instvar choices_;
    set current_ "";
    set choices_ "";
    eval $self next $args;
}

ExclusiveChoiceParameter instproc get {} {
    $self instvar current_;

    return $current_;
}

ExclusiveChoiceParameter instproc set value {
    $self instvar current_ choices_

    if {[lsearch $choices_ $value] != -1} {
	set current_ $value;
	return $current_;
    }
}

ExclusiveChoiceParameter instproc type {} {
    return "exclusive_choice";
}

ExclusiveChoiceParameter instproc domain {} {
    $self instvar choices_;
    return $choices_;
}

ExclusiveChoiceParameter instproc add {new_choice} {
    $self instvar choices_;

    lappend choices_ $new_choice;
}

Class RectSubregionParameter -superclass Parameter

RectSubregionParameter instproc init args {
    $self instvar current_

    set current_ "0.25 0.25 0.75 0.75";
    eval $self next $args;
}

RectSubregionParameter instproc get {} {
    $self instvar current_;
    
    return $current_;
}

RectSubregionParameter instproc set value {
    $self instvar current_;
    
    set current_ $value;
    return $current_;
}

RectSubregionParameter instproc type {} {
    return "rect_subregion";
}

RectSubregionParameter instproc domain {} {
    return "all"
}

