

#XXX: change working directory to mash so that relative paths will be correct
if {$tcl_platform(platform) == "windows"} {
	if ![file exists mash.exe] {		
		if [file exists mash\\mash.exe] {
			puts "changing dir to mash"
			cd mash
		}
	} 
}



Class Setup/MASH -superclass Setup

Setup/MASH public init { } {
	$self next
	$self set appname_ "MASH"
	$self set copyright_ "lib/setup/COPYRIGHT"
	if { [Setup system]=="Win32" } {
		$self set destination_ "C:/Program Files/mash"
	}

	$self set components_(typical) [list "mash binaries" scripts \
			"active services" mplug]
	$self set components_(custom) [$self set components_(typical)]
	$self set {component_help_(mash binaries)} \
			"This installs the executable files for\
			\nthe MASH shell and the Session Directory\
			\ntool"
	$self set component_help_(scripts) \
			"These are the various scripts for multimedia\
			\nconferencing that run on the MASH platform"
	$self set {component_help_(active services)} \
			"These are the various scripts for the Active\
			\nServices Infrastructure that is built on the\
			\nMASH platform"
	$self set component_help_(mplug) \
			"This is a Web browser plugin for the MASH\
			\nplatform. It includes the plugin shared\
			\nlibrary and helper scripts and initialization\
			\nfiles"

	if { [Setup system]=="Win32" } {
		$self set {files_(mash binaries)} \
				[list mash.exe rsdr.exe tclpip80.dll]
	} else {
		global mash
		$self set {files_(mash binaries)} \
				[list bin/mash-$mash(version) \
				bin/smash-$mash(version) bin/rsdr]
	}

	if { [Setup system]=="Win32" } {
		$self set files_(scripts) \
				[list vic vat mb recorder player \
				vic.exe vat.exe mb.exe \
				recorder.exe player.exe \
				collaborator.exe mbv2.exe \
				nsdr.exe \
				lib/setup/setup lib/setup/setup.gif \
				lib/setup/COPYRIGHT]
	} else {
		$self set files_(scripts) \
				[list bin/vic bin/vat bin/mb \
				bin/recorder bin/player \
				bin/collaborator bin/mbv2 bin/nsdr \
				lib/setup/setup lib/setup/setup.gif \
				lib/setup/COPYRIGHT]
	}

	if { [Setup system]=="Win32" } {
		$self set {files_(active services)} \
				[list as/hm as/vgw as/agw as/sdgw as/mars \
				as/tgmb as/pathfinder as/megafor \
				as/mgamon as/mediapad.prc \
				as/hm.exe as/vgw.exe as/agw.exe as/sdgw.exe \
				as/mars.exe as/tgmb.exe as/pathfinder.exe \
				as/megafor.exe as/mgamon.exe \
				as/html/*.html as/html/images/*.gif]
	} else {
		$self set {files_(active services)} \
				[list as/hm as/vgw as/agw as/sdgw as/mars \
				as/tgmb as/pathfinder as/megafor \
				as/mediapad.prc \
				as/mgamon as/html/*.html as/html/images/*.gif]
	}

	$self instvar files_	
	set files_(mplug) \
			 [list lib/npmplug* lib/prefs-mplug \
			 lib/mplug-test.html lib/mplug-test.mash]
	if  { [Setup system]=="Win32" } {
		lappend files_(mplug) script-loader 
	} else {
		lappend files_(mplug) bin/script-loader
	}
}

Setup/MASH public dialog_list { } {
	set list [$self next]
	lappend list SetEnvironDlg InstallMPlugDlg
	return $list
}


Setup/MASH public do_install { } {
	SetupProgress/MASH .progress
	.progress center
	
	$self copy_files
	# on unix, create a symlink from mash to mash-$mash(version)
	$self link_mash
	.progress check_next
	update
	
	$self update_environment
	.progress check_next
	update
	
	$self install_mplug
	.progress check_next
	update
	
	destroy .progress
}


Setup/MASH public success { } {
	$self instvar appname_ destination_
	set text "\n$appname_ has been successfully installed\
			\on your system.\n"

	if {[Setup system]=="Win32"} {
		set bin [file nativename $destination_]
	} else {
		set bin [file nativename [file join $destination_ bin]]
	}
	set lib [file nativename [file join $destination_ lib]]
	append text "\nYou can find the executable files associated with\
			\n$appname_ in \"$bin\".\
			\nThe libraries, examples and initialization files\
			\nfor the various applications are in\
			\n\"$lib\"\n\
			\nEnjoy playing around with your new software!!!\n\n"

	set retval [Dialog transient MessageBox -text $text -image Icons(cal) \
			-type \
			{ {readme -text "View README.TXT" -under 0 \
			-image Icons(textfile) } \
			{ exit -text "Exit" -under 1 -image Icons(cross) } }]
	if { $retval == "readme" } {
		ReadmeDlg .readmedlg
		.readmedlg invoke
	}
}


Setup/MASH private link_mash { } {

	if { [Setup system]!="Unix" } return
	
	$self instvar destination_
	global mash
	set to [file join $destination_ bin/mash]
	$self copy_or_symlink/[Setup system] mash-$mash(version) $to

	set to [file join $destination_ bin/smash]
	$self copy_or_symlink/[Setup system] smash-$mash(version) $to
}


Setup/MASH private update_environment { } {
	$self instvar update_user_environ_ update_system_environ_

	.progress subwidget bar configure -max 2 -value 0
	update
	
	if { $update_user_environ_==1 } {
		.progress subwidget info configure -text \
				"Updating user environment"
		update
		$self update_user_environment/[Setup system]
	}
	.progress subwidget bar configure -value 1
	# no need to do an update here
	
	if { $update_system_environ_==1 } {
		.progress subwidget info configure -text \
				"Updating system environment"
		update
		$self update_system_environment/[Setup system]
	}
	.progress subwidget bar configure -value 2
	update
}




Setup/MASH private update_user_environment/Unix {} {
	# we assume that $HOME will always be defined under UNIX
	# so all we need to set is $MASH_HOME

	$self instvar destination_

	QueryUnixRCFilesDlg .query
	set list [.query invoke]

	set comment "\n\n# Environment variables for MASH\n"
	set envline(.cshrc)   "${comment}setenv MASH_HOME $destination_"
	set envline(.tcshrc)  $envline(.cshrc)
	set envline(.profile) "${comment}MASH_HOME=$destination_\
			\nexport MASH_HOME"

	set shells(.cshrc)   "csh and tcsh"
	set shells(.tcshrc)  "tcsh"
	set shells(.profile) "sh and bash"

	foreach rcfile $list {
		$self update_environment_file "~/$rcfile" $envline($rcfile) \
				$shells($rcfile)
	}
}


Setup/MASH private update_system_environment/Unix {} {
	# we assume that $HOME will always be defined under UNIX
	# so all we need to set is $MASH_HOME

	$self instvar destination_

	$self update_environment_file /etc/csh.cshrc \
			"setenv MASH_HOME $destination_" "csh and tcsh"
	$self update_environment_file /etc/profile \
			"MASH_HOME=$destination_\nexport MASH_HOME" \
			"sh and bash"
}


Setup/MASH private update_environment_file { filename envline shells } {
	if [file exists $filename] {
		if { [catch { set file [open $filename a] } msg ] } {
			set file ""
		}
	} else {
		if { [catch { set file [open $filename w] } msg ] } {
			set file ""
		}
	}

	if { $file=="" } {
		$self warn "Could not open $filename:\n$msg\n\nUnable to \
				update environment for $shells"
		# proceed on
	} else {
		puts $file $envline
		close $file
	}
}


Setup/MASH private update_user_environment/Win32 { } {
	global tcl_platform
	if { $tcl_platform(os)=="Windows 95" } {
		$self update_user_environment/Win95
	} else {
		$self update_user_environment/WinNT
	}
}


Setup/MASH private update_system_environment/Win32 { } {
	global tcl_platform
	if { $tcl_platform(os)=="Windows NT" } {
		$self update_system_environment/WinNT
	}
}


Setup/MASH private update_user_environment/Win95 { } {
	$self instvar destination_

	set dest [file nativename $destination_]
	$self update_environment_file C:/autoexec.bat \
			"set MASH_HOME=$dest" "Windows 95"
	global env
	if { ![info exists env(HOME)] || $env(HOME)=="" } {
		set def_home "C:\\"
		$self update_environment_file C:/autoexec.bat \
				"set HOME=\"$def_home\"" "Windows 95"

		Dialog transient MessageBox -text "\
				\nSetup has set your HOME environment variable\
				\nto ${def_home}. It is recommended that you\
				\nchange this to a directory where you store\
				\nyour local files.\n\
				\nSetup will look for a directory \".mash\" in\
				\nthis sub-directory for user-customized\
				\noptions.\n\
				\nYou can update your environment settings by\
				\nmanually editing the C:\\autoexec.bat file\
				\n\n" -image Icons(warning) -type ok
	}
}


Setup/MASH private update_user_environment/WinNT { } {
	$self instvar destination_

	set dest [file nativename $destination_]
	putregistry "HKEY_CURRENT_USER\\Environment" "MASH_HOME" $dest
	if { [catch {set HOME [getregistry \
			"HKEY_CURRENT_USER\\Environment" HOME]}] || \
			$HOME=={} } {
		# couldn't find a valid HOME env var
		# so, set it
		set def_home "C:\\"
		putregistry "HKEY_CURRENT_USER\\Environment" "HOME" $def_home

		Dialog transient MessageBox -text "\
				\nSetup has set your HOME environment variable\
				\nto ${def_home}. It is recommended that you\
				\nchange this to a directory where you store\
				\nyour local files.\n\
				\nSetup will look for a directory \".mash\" in\
				\nthis sub-directory for user-customized\
				\noptions.\n\
				\nYou can update your environment settings by\
				\nselecting \"System\" from the Control Panel\
				\nand choosing the \"Environment\" tab.\n\n" \
				-image Icons(warning) -type ok
	}
}


Setup/MASH private update_system_environment/WinNT { } {
	$self instvar destination_
	
	set dest [file nativename $destination_]
	set key "HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\"
	append key "Control\\Session Manager\\Environment"

	putregistry $key "MASH_HOME" $dest
	if { [catch {set HOME [getregistry $key HOME]}] || $HOME=={} } {
		# couldn't find a valid HOME env var
		# so, set it
		set def_home "C:\\"
		putregistry $key "HOME" $def_home

		Dialog transient MessageBox -text "\
				\nSetup has set a system-wide HOME environment\
				\nvariable to ${def_home}. It is recommended\
				\nthat you change this on a per-user basis\
				\nto a directory where the user stores his/her\
				\nlocal files.\n\
				\nSetup will look for a directory \".mash\" in\
				\nthis sub-directory for user-customized\
				\noptions.\n\
				\nYou can update your environment settings by\
				\nselecting \"System\" from the Control Panel\
				\nand choosing the \"Environment\" tab.\n\n" \
				-image Icons(warning) -type ok
	}
}


Setup/MASH private install_mplug { } {
	$self instvar install_components_
	if { [lsearch $install_components_ mplug] == -1 } {
		# mplug has not been installed
		return
	}
	
	$self instvar destination_

	set npmplug_so [file join $destination_ lib/npmplug.*]
	if { [catch { set npmplug_so [glob $npmplug_so] } msg] } {
		$self warn "Could not find MASH plugin files:\n$msg\n\
				\nSome components seem to be missing from the\
				\ninstallation package."
		return
	}

	$self instvar mplug_dest_
	
	.progress subwidget bar configure -max [llength $mplug_dest_] \
			-value 0
	.progress subwidget info configure -text \
			"Installing MASH plugin"
	update
	set done 0
	
	foreach dir $mplug_dest_ {
		set msg [$self create_dir $dir]
		if { $msg!={} } {
			$self warn "Could not create directory:\
					$dir\n$msg"
				# don't return here; go on with the remaining
				# dirs
		}

		foreach file $npmplug_so {
			$self copy_or_symlink/[Setup system] $file $dir
		}

		incr done
		.progress subwidget bar configure -value $done
		update
	}

	# now update the prefs-mplug file
	.progress subwidget info configure -text \
			"Updating preferences file for MASH plugin"
	update
	$self update_prefs_mplug
}


Setup/MASH private copy_or_symlink/Unix { src dest } {
	set src  [$self do_tilde_subst $src]
	set dest [$self do_tilde_subst $dest]
	
	if {[catch { set retval [exec ln -sf "$src" "$dest"] } msg ] } {
		$self warn "Could not create symbolic links between\
				\n$src and $dest:\n$msg"
		return
	} else {
		set retval [string trim $retval]
		if { $retval!={} } {
			$self warn "Possible error while creating symbolic\
					links\nbetween $src and $dest\n:\
					\n$retval"
			return
		}
	}
}


Setup/MASH private copy_or_symlink/Win32 { src dest } {
	if {[catch { file copy -force -- $src $dest } msg ] } {
		$self warn "Could not copy MASH plugin files\
				\n$src to $dest"
		return
	}
}


Setup/MASH private do_tilde_subst { file } {
	# XXX: we are only checking for leading ~/
	set split [file split $file]
	if { [lindex $split 0] == "~" } {
		global env
		if [info exists env(HOME)] {
			set file [eval file join \
					[lreplace $split 0 0 $env(HOME)]]
		} else {
			set file [eval file join [lreplace $split 0 0]]
		}
	}

	return $file
}


Setup/MASH private update_prefs_mplug { } {
	$self instvar destination_
	set prefs_mplug [file join $destination_ "lib/prefs-mplug"]

	# open the preferences file
	
	if { ![file exists $prefs_mplug] } {
		$self warn "Could not find file \"$prefs_mplug\"\
				\nSome components seem to be missing from the\
				installation\npackage."
		return
	}

	if { [catch {set file [open $prefs_mplug]} msg] } {
		$self warn "Error occurred while opening the MASH plugin\
				\npreferences file \"$prefs_mplug\"\n($msg)\
				\n\nSetup will not be able to update the\
				\plugin preferences."
		return
	}

	# replace all occurrences of $MASH_HOME with the actual path
	set mash_home [file nativename $destination_]
	set buffer ""
	while { [gets $file line] >= 0 } {
		set line [string trim $line]
		set first [string index $line 0]
		if { $first!="#" } {
			# this line is not a comment line

			regsub -all -- {\$MASH_HOME} $line $mash_home line
		}

		append buffer "$line\n"
	}
	close $file
	
	# output the new mplug-prefs file
	if { [catch {set file [open $prefs_mplug w]} msg] } {
		$self warn "Error occurred while writing to the MASH plugin\
				\npreferences file \"$prefs_mplug\"\n($msg)\
				\n\nSetup will not be able to update the\
				\plugin preferences."
		return
	}

	if { [catch {puts -nonewline $file $buffer} msg] } {
		$self warn "Error occurred while writing to the MASH plugin\
				\npreferences file \"$prefs_mplug\"\n($msg)\
				\n\nSetup will not be able to update the\
				\plugin preferences."
		return
	}
	
	close $file
}




WidgetClass SetEnvironDlg -superclass SetupDialog -default {
	{ .title "Update Environment Variables" }
	{ *Checkbutton.borderWidth 1 }
}


SetEnvironDlg private build_widget { path } {
	$self tkvar update_user_environ_ update_system_environ_
	
	set path [$self build_skeleton $path [list Back Next Cancel]]
	label $path.label -justify left -text \
			"Setup will attempt to reconfigure you environment\
			\nby adding the appropriate system variables to your\
			\nstartup files\
			\n\nSetup can reconfigure the environment for either,\
			\njust you (User Environment), or the entire\
			\nsystem (System Environment), or both. You must have\
			\nadministrative/root priviledges in order to update\
			\nthe System Enviroment. Check that box only if you\
			\nhave those priviledges.\
			\n\nOn UNIX systems, you can get root priviledges by\
			\nrunning Setup as the \"root\" user. On Windows\
			\nsystems, you must be the Administrator in order to\
			\nupdate the System Environment\
			\n\nYou can choose to skip this step completely by not\
			\nselecting either of the following choices.\n"

	frame $path.frame -borderwidth 2 -relief groove
	label $path.envlabel -text "Update environment variables"

	set update_user_environ_ 1
	set update_system_environ_ 0
	checkbutton $path.user -text "User Environment (only yours)" \
			-variable [$self tkvarname update_user_environ_]
	checkbutton $path.system -text "System Environment (global)" \
			-variable [$self tkvarname update_system_environ_]

	global tcl_platform
	if { $tcl_platform(os)=="Windows 95" } {
		# there is no such thing as system/user environment on Win95
		$path.system configure -state disabled
	}
	
	pack $path.label -side top -pady 5 -padx 5
	pack $path.envlabel -in $path.frame -side top -anchor w -padx 5 -pady 2
	pack $path.user   -in $path.frame -side top -anchor w -padx 20 -pady 1
	pack $path.system -in $path.frame -side top -anchor w -padx 20 -pady 1
	pack $path.frame -side top -padx 5 -pady 10 -fill x
}


SetEnvironDlg private verify { direction } {
	if { $direction=="back" } { return 1 }

	# update the corresponding variables in the Setup object
	set setup [Setup instance]

	$self tkvar update_user_environ_ update_system_environ_
	$setup set update_user_environ_ $update_user_environ_
	$setup set update_system_environ_ $update_system_environ_

	return 1
}


WidgetClass InstallMPlugDlg -superclass SetupDialog -default {
	{ .title "Copy MASH plugin files" }
	{ *Checkbutton.borderWidth 1 }
}


InstallMPlugDlg private build_widget { path } {
	set setup [Setup instance]
	$setup instvar appname_

	set path [$self build_skeleton $path [list Back Next Cancel]]
	set text \
			"The $appname_ software comes with a plugin for\
			Netscape and\nMicrosoft Internet Explorer that can be\
			used to enhance\nthe capabilities of your web-browser\
			by allowing\nit to browse the MBone and participate\
			in MBone sessions\
			\n\nSetup must copy the plugin files to your browser's\
			directory,\nin order for it to work.\n\
			\nThe Setup program has auto-detected the following\
			browser\nplugin directories on your system. Please\
			select \"Add to list\"\
			\nto make additions to this list.\n\n"

	if { [Setup system]=="Unix" } {
		append text \
				"Your browser's plugin directory is usually a\
				directory called\n\"plugins\" somewhere under\
				your browser's installation.\
				\nFor example,\
				/usr/local/netscape/lib/plugins\n\
				\nPlease note that you might require root\
				priviledges in order to\ninstall the plugin\
				in some of these directories.\n"
	} else {
		# win32
		append text \
				"Your browser's plugin directory is usually a\
				directory called \"Plugins\" somewhere under\
				\nyour browser's installation.\
				\nFor example,\
				\n    C:\\Program Files\\Netscape\\Communicator\\Program\\Plugins, or\
				\n    C:\\Program Files\\Microsoft Internet\\Plugins\n\
				\nPlease note that you might require administrative\
				priviledges in order to\ninstall the plugin\
				in some of these directories.\n"
	}

	label $path.label -justify left -text $text
	pack $path.label -side top -pady 5 -padx 5

	frame $path.f1 -borderwidth 2 -relief groove
	frame $path.f2
	label $path.cplabel -text "Copy MASH plugin files to:" -justify left \
			-anchor w
	pack $path.cplabel -in $path.f2 -side left -anchor w -padx 5 -pady 0 \
			-fill x -expand 1

	button $path.add -text "Add to list..." -command "$self add"
	pack $path.add -in $path.f2 -side right -anchor e -padx 5 -pady 0
	pack $path.f2 -in $path.f1 -padx 0 -pady 2 -fill x

	$self tkvar copy_list_
	set copy_list_(count) 0
	foreach mplugpath [$self search_browsers] {
		set copy_list_(path,$mplugpath) 0
		checkbutton $path.check$copy_list_(count) -text $mplugpath \
				-variable \
				[$self tkvarname copy_list_(path,$mplugpath)]
		pack $path.check$copy_list_(count) -in $path.f1 -side top \
				-anchor w -padx 20 -pady 1
		incr copy_list_(count)
	}

	pack $path.f1 -side top -padx 5 -pady 10 -fill x

	label $path.disabled -justify left -anchor w -text \
			"The installation that you have chosen does not\
			\ninclude the MASH plugin. If you would like to\
			\ninstall the plugin, please hit \"Back\" and\
			\nchange your installation options."
	# don't pack this label here
	# pack it in "invoke" only if mplug has been disabled
}


InstallMPlugDlg private verify { direction } {
	if { $direction=="back" } { return 1 }

	$self tkvar copy_list_
	set setup [Setup instance]
	$setup instvar mplug_dest_
	set mplug_dest_ {}
	foreach name [array names copy_list_ "path,*"] {
		if { $copy_list_($name) } {
			set filepath [string range $name 5 end]
			lappend mplug_dest_ $filepath
		}
	}
	
	return 1
}


InstallMPlugDlg public invoke { args } {
	# check if MPlug is part of the components list that we are trying to
	# install

	set setup [Setup instance]
	$setup instvar install_components_
	if { [lsearch $install_components_ mplug] == -1 } {
		# disable this dialog box
		$self set_state disabled
	} else {
		$self set_state normal
	}
	return [eval [list $self] next $args]
}


InstallMPlugDlg private set_state { state } {
	$self tkvar copy_list_
	set path [$self subwidget main]
	for { set idx 0 } { $idx < $copy_list_(count) } { incr idx } {
		 $path.check$idx configure -state $state
	}
	$path.add configure -state $state
	if { $state=="disabled" } {
		set fg [$path.add cget -disabledforeground]
		pack $path.disabled -side top -padx 5 -pady 5 -anchor w
	} else {
		set fg [$path.add cget -foreground]
		catch {pack forget $path.disabled}
	}
	$path.label configure -foreground $fg
	$path.cplabel configure -foreground $fg
}


InstallMPlugDlg private search_browsers { } {
	return [$self search_browsers/[Setup system]]
}


InstallMPlugDlg private add { } {
	$self tkvar copy_list_
	set setup [Setup instance]
	set dirdlg [$setup get_dirdlg]

	$dirdlg configure -title "Choose browser plugin directory"
	$dirdlg subwidget dirbox configure -allownonexistent 0

	set retval [$dirdlg invoke]
	if { $retval!={} } {
		set path [$self info path]
		set copy_list_(path,$retval) 1
		checkbutton $path.check$copy_list_(count) -text $retval \
				-variable \
				[$self tkvarname copy_list_(path,$retval)]
		pack $path.check$copy_list_(count) -in $path.main.f1 \
				-side top \
				-anchor w -padx 20 -pady 1
		incr copy_list_(count)
	}
}



InstallMPlugDlg private search_browsers/Unix { } {
	# return a list of directories; look for a default
	# /usr/local/netscape/lib/plugins or
	# /usr/local/lib/netscape/plugins

	set list [list ~/.netscape/plugins]
	foreach dir [list /usr/local/netscape/lib/plugins \
			/usr/local/lib/netscape/plugins] {
		if [file isdirectory $dir] { lappend list $dir }
	}
	
	return $list
}


InstallMPlugDlg private search_browsers/Win32 { } {
	# return a list of directories

	set list {}

	# first try to locate the default browser
	set key "HKEY_LOCAL_MACHINE\\Software\\Classes\\"
	append key "http\\shell\\open\\command"
	if { ![catch {set browser [getregistry $key ""]}] && $browser!={} } {
		# extract the name of the browser from the data
		# the filename might contain spaces, so I can't just
		# use [lindex $browser 0] to get the filename
		#
		# The stupid thing is that Windows does not necessarily
		# quote the pathname, so I'm just going to use a heuristic
		# where if there is a word that starts with a -, I'll assume
		# that it's the start of the arglist and assume everything
		# before that is the pathname
		# XXX:
		
		set browser [string trim $browser]
		set idx [string first " -" $browser]
		if { $idx==-1 } { set idx [string first " \"-" $browser] }
		if { $idx!=-1 } {
			set pathname [string trim [string range $browser 0 \
					[expr $idx-1]]]
		} else {
			set pathname $browser
		}

		# check if the Plugins directory exists
		set plugindir [file join [file dirname $pathname] Plugins]
		if [file isdirectory $plugindir] {
			set list [list $plugindir]
		}
	}

	foreach dir [list "C:/Program Files/Netscape/Communicator/Program/Plugins" "C:/Program Files/Netscape/Navigator/Program/Plugins" "C:/Program Files/Microsoft Internet/Plugins" "C:/Program Files/Plus!/Microsoft Internet/Plugins" "C:/Program Files/Internet"] {

		if [file isdirectory $dir] {
			# first check if it's already there
			if { [lsearch [string tolower $list] \
					[string tolower $dir]] == -1 } {
				lappend list $dir
			}
		}
	}

	return $list
}


WidgetClass QueryUnixRCFilesDlg -superclass Dialog -default {
	{ .title "Select RC files" }
	{ *Checkbutton.borderWidth 1 }
	{ *ImageTextButton.borderWidth 1 }
	{ *Label.font WidgetDefault }	
}


QueryUnixRCFilesDlg private build_widget { path } {
	label $path.label -justify left -anchor w -text \
			"Setup can update one or more of the following startup\
			\nfiles in your home directory. Please select which\
			\nones you would like Setup to update, depending on\
			\nyour preferred shell"
	checkbutton $path.cshrc -text ".cshrc (csh and tcsh)" -anchor w \
			-variable [$self tkvarname cshrc]
	checkbutton $path.tcshrc -text ".tcshrc (tcsh only)" -anchor w \
			-variable [$self tkvarname tcshrc]
	checkbutton $path.profile -text ".profile (sh and bash)" -anchor w \
			-variable [$self tkvarname profile]
	ImageTextButton $path.ok -text "Ok" -image Icons(check) \
			-command "$self ok"

	pack $path.label -side top -fill x -padx 10 -pady 10
	pack $path.cshrc $path.tcshrc $path.profile -fill x -padx 20
	pack $path.ok -anchor e -padx 20 -pady 15
}


QueryUnixRCFilesDlg private ok { } {
	$self tkvar cshrc tcshrc profile
	set list {}
	if { $cshrc   } { lappend list .cshrc   }
	if { $tcshrc  } { lappend list .tcshrc  }
	if { $profile } { lappend list .profile }

	$self configure -result $list
}


WidgetClass SetupProgress/MASH -superclass SetupProgress


SetupProgress/MASH private checklist { } {
	set list [$self next]
	lappend list "Update environment variables" "Install MASH plugin"
	return $list
}



WidgetClass ReadmeDlg -superclass SetupDialog -default {
	{ .title "README.TXT" }
	{ *Text*borderWidth 1 }
	{ *Scrollbar*borderWidth 1 }
	{ *Scrollbar*width 10 }
}


ReadmeDlg private build_widget { path } {
	if [catch {set file [open README.TXT "r"]} msg ] {
		set msg "Could not open README file:\n$msg"
	} else {
		set msg ""
		while { [gets $file line] >= 0 } {
			append msg "$line\n"
		}
		close $file		
	}
	
	set path [$self build_skeleton $path [list Exit]]
	ScrolledText $path.readme -scrollbar both -options \
			{ {text.width 60} {text.height 16} {text.wrap none} }
	$path.readme subwidget text insert end $msg
	$path.readme subwidget text configure -state disabled
	pack $path.readme -fill both -expand 1
}


